//-----------------------------------------------------
// Pic Pack library
// 
// ea_its_temp_sensor.c
//
//
// (c) Ian Harris 2010
// imharris [at] gmail.com
//
// 
//-----------------------------------------------------


// Configuration of the system
#include "config.h"
// Information about our platform
#include "platform.h"
// Leds on our platform
#include "platform_leds.h"
// General utilities
#include "pic_utils.h"
// Serial port support
#include "pic_serial.h"
// MRF24J40 support
#include "mrf24j40.h"
// SPI hardware module support
#include "spi_hw.h"
// WPAN (802.15.4)
#include "wpan.h"
// Timer 0 module
#include "pic_timer.h"
// Turn timer overflows into ticks
#include "pic_tick.h"
// Terminal 
#include "pic_term.h"
// ITS mode 2
#include "its_mode2.h"
// ITS common networking support
#include "its_common.h"
// Software i2c support
#include "i2c.h"
// TMP75 temperature sensor
#include "tmp75.h"

// System includes
#include "string.h"
#include "stdlib.h"


bit temp_requested = 0;
uns16 request_id;

bit promiscuous_mode = 0;


// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	
	serial_handle_tx_isr();
	serial_handle_rx_isr();
	timer_handle_0_isr();	
}


	



// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs
	
	// Platform leds ----------------------------
	
	platform_leds_setup_io();
	
	// initialise ITS networking ----------------------------
	
	// mrf INT pin
	
	clear_bit(intcon, INT0IF);	// in case it was triggered already
	clear_bit(intcon2, INTEDG0);	// Need falling edge for mrf24j40 defaults
	//!!set_bit(intcon, INT0IE);	// turn on rb0/int0 interrupts
	
	// Hardware SPI module
	
	spi_hw_setup_io();
	spi_hw_init();
	
	// MRF module itself
	
	mrf24j40_setup_io();
	mrf24j40_init();
	mrf24j40_set_channel(ITS_FIXED_CHANNEL);

	// timer is used for ITS networkign
	
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 5000);	// 1ms at 20Mhz

	// TMP75 temp sensor ----------------------------

	tmp75_setup_io();
	// 12 bit temp, shutdown
	// OS R1 R0 F1 F0 POL TM SD
	// 0  1  1  0  0  0   0  0   
	tmp75_set_config(TMP75_ADDR, 0b01100000);
	
	// Serial port and terminal ----------------------------
	
	serial_setup(SPBRG_115200);
	term_init();
	
	// Button  ----------------------------
	
	make_input(PORTA, 4);
	
	// General init ----------------------------
	
	turn_peripheral_ints_on();
	turn_global_ints_on();
}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void term_entry_callback(uns8 *term_buffer) {
	
	uns8 data[10];
	uns8 EA_1[8] = { 0, 0, 0, 0, 0, 0, 0, 1 };
	uns8 EA_2[8] = { 0, 0, 0, 0, 0, 0, 0, 2 };
	uns8 EA_3[8] = { 0, 0, 0, 0, 0, 0, 0, 3 };
	
	uns8 first_char = term_buffer[0];
	
	switch (first_char) {
		case '1':
			serial_print_str("TX1... ");
			data[0] = 'T';
			data[1] = 'O';
			// we just happen to know the device is 0x2020
			its2_router_queue_packet(0x0001, ITS_GENERIC_DATA, &data, 2, 1);

			break;
		case '2':
			serial_print_str("TX2.... ");
			data[0] = 'o';
			data[1] = 't';
			its2_router_queue_packet(0x0002, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case '3':
			serial_print_str("TX3.... ");
			data[0] = 'T';
			data[1] = 't';
			its2_router_queue_packet(0x0003, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 'a':
			serial_print_str("Setting address to 1\n");
			mrf24j40_set_extended_address(&EA_1);
			its2_router_init(0x0001, ITS_NETWORK_ID);
			
			break;
		case 'b':
			serial_print_str("Setting address to 2\n");
			mrf24j40_set_extended_address(&EA_2);
			its2_router_init(0x0002, ITS_NETWORK_ID);
			break;
		case 'c':
			serial_print_str("Setting address to 3\n");
			mrf24j40_set_extended_address(&EA_3);
			its2_router_init(0x0003, ITS_NETWORK_ID);
			break;
		case 'p':
			serial_print_str("Setting pan id = 5\n");
			mrf24j40_set_pan_id(0x05);
			break;
		case 'r':
			if (promiscuous_mode) {
				serial_print_str("Prom OFF\n");
				promiscuous_mode = 0;
				mrf24j40_short_addr_write(RXMCR, 0x00); 
			} else {
				promiscuous_mode = 1;
				serial_print_str("Prom ON\n");
				mrf24j40_short_addr_write(RXMCR, 0x01); // promiscuous mode, no coord
			}
			break;
		case 'd':
				serial_print_str("debug mode\n");
				debug_module = 1;
				uns8 config = tmp75_get_config(TMP75_ADDR);
				serial_print_str("config = ");
				serial_print_int_hex(config);
				serial_print_nl();
				uns16 temp = tmp75_get_temp(TMP75_ADDR);
				serial_print_str("Temp = ");
				serial_print_int_hex_16bit(temp);
				serial_print_nl();


				break;
		case 'i':
				its_print_devices();
				break;
			
		case 'q':
			its2_print_queue();
			break;
	} 	// switch		

}

void its2_router_receive_callback(uns16 device_id, uns8 *data, uns8 data_length) {
		
		platform_leds_flash(1);
		
		serial_print_str("Controller receives from ");
		serial_print_int(device_id);
		serial_print_nl();
		request_id = device_id;
		temp_requested = 1;
		//ds1631_convert_temp(DS1631_ADDR);	
}

void its2_transmit_status_callback(its2_packet *pkt, uns8 status) {
	serial_print_str("{TX src:");
	serial_print_int(pkt->its_source_id);
	serial_print_str(" dst: ");
	serial_print_int(pkt->its_dest_id);
	serial_print_str(" stat=");
	switch (status) {
		case ITS2_TX_STATUS_SUCCESS:
			serial_print_str("Success");
			break;
		case ITS2_TX_STATUS_NO_ROUTE:
			serial_print_str("No route");
			break;
		case ITS2_TX_STATUS_NO_ACK:
			serial_print_str("No ack");
			break;
		case ITS2_TX_STATUS_NEXT_HOP_UNKNOWN:
			serial_print_str("Next hop unknown");
			break;
	}
	serial_print_str("} ");
	
	if (status == ITS2_TX_STATUS_SUCCESS) {
		platform_leds_flash(2);
	} else {
		platform_leds_flash(3);
	}	
}			


void main() {

	uns8 my_ea[8] = { 0, 0, 0, 0, 0, 0, 0, 2};
	
	configure_system();

	delay_ms(100);
	delay_ms(100);
	
	serial_print_str("\n\nITS M2 Temperature Sensor\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

//	its2_router_init(/* device */ 0x1010, /* network id */ 0x5000);
	serial_print_str("Setting address to 3\n");
	mrf24j40_set_extended_address(&my_ea);
	its2_router_init(0x0002, ITS_NETWORK_ID);
	
	timer_start_0();
	
	uns16 temp;
	
	for(;;) {

		if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		mrf24j40_handle_isr();
		}

		wpan_process();	

		its2_router_process();
		//set_bit(intcon, INT0IE);
		
		term_process();
		
		platform_leds_process();	// See if any need turning off

		if (temp_requested) {	// temp requested, so lets see if it has finished
			uns8 config = 0; //ds1631_get_config(DS1631_ADDR);
			//if (test_bit(config, 7)) {	// the DONE bit
				temp_requested = 0;
				temp = tmp75_get_temp(TMP75_ADDR);
				serial_print_str("Temp = ");
				serial_print_int_hex_16bit(temp);
				serial_print_nl();
				its2_router_queue_packet(request_id, ITS_GENERIC_DATA, (uns8*)&temp, 2, ITS2_FLAG_ACK);

		}
		
		if (!test_pin(PORTA, 4)) {
			serial_putc('B');
		}	


	}	// while (1)

	
}	// main

