//-----------------------------------------------------
// Pic Pack library
// 
// its_m2_router.c
//
//
// (c) Ian Harris 2010
// imharris [at] gmail.com
//
// 
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "mrf24j40.h"
#include "spi_hw.h"
#include "wpan.h"
#include "pic_timer.h"
#include "pic_tick.h"

#include "its_mode2.h"
#include "its_common.h"

// System includes
#include "string.h"
#include "stdlib.h"

#include "convert.h"

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;

// Dummy variable to show we can set it 
uns8 send_to = 0;

bit promiscuous_mode = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
	if (test_bit(intcon, INT0IF)) {
		clear_bit(intcon, INT0IF);
		mrf24j40_handle_isr();
	}	
	serial_handle_tx_isr();
	serial_handle_rx_isr();
	timer_handle_0_isr();	
}


// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed
	
	if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
		boostbloader();
	}

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case '1':
				do_task = 1;
				break;
		    case '2':
				do_task = 2;
				break;
		    case '3':
				do_task = 9;
				break;
			case 'c':
				do_task = 3;
				break;
			case 'd':
				do_task = 4;
				break;
			case 'e':
				do_task = 8;
				break;
			case 'p':
				do_task = 5;
				break;	
			case 'r':
				do_task = 6;
				break;								
			case 't':
				do_task = 7;
				break;
			case 'q':
				do_task = 10;
				break;
			case 'h':
				do_task = 11;
			default:
				// More complicated example
				// Let's set a variable based on input
				// You can type:
				// s05
				// And press enter, it will set "send_to" to this (in hex)
				if ((first_char == 's') && (strlen(serial_buffer)==3)) {
					send_to = atoui_hex(&serial_buffer[1]);
					serial_print_str(" Set send_to to ");
					serial_print_int(send_to);	// Of course, this could be anything
					serial_putc('\n');
				};
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs

	clear_bit(intcon, INT0IF);	// in case it was triggered already
	clear_bit(intcon2, INTEDG0);	// Need falling edge for mrf24j40 defaults
	set_bit(intcon, INT0IE);	// turn on rb0/int0 interrupts
	
	its_init();

	#ifdef _PIC16
		timer_setup_0(TIMER_8BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xff - 250 - 1);	// 1ms at 8Mhz
	#endif

	#ifdef _PIC18
		timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 16000);	// 1ms at 64Mhz
		timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 5000);	// 1ms at 20Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_2, 0xffff - 1500 + 7);	// 1ms at 12Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xffff - 375 + 1);	// 1ms at 12Mhz
	#endif	

		
	serial_setup(SPBRG_115200);

	turn_peripheral_ints_on();
	turn_global_ints_on();
}


// Handle any command line tasks we need to do
// Very handy to print out variables, do commands,
// print out some debug, etc...

void handle_tasks() {
	
	uns8 data[10];
	uns8 EA_1[8] = { 0, 0, 0, 0, 0, 0, 0, 1 };
	uns8 EA_2[8] = { 0, 0, 0, 0, 0, 0, 0, 2 };
	uns8 EA_3[8] = { 0, 0, 0, 0, 0, 0, 0, 3 };
	
	switch (do_task) {
		case 1:
			serial_print_str("TX1... ");
			data[0] = 'T';
			data[1] = 'O';
			// we just happen to know the device is 0x2020
			its2_router_queue_packet(0x0001, ITS_GENERIC_DATA, &data, 2, 1);

			break;
		case 2:
			serial_print_str("TX2.... ");
			data[0] = 'o';
			data[1] = 't';
			its2_router_queue_packet(0x0002, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 9:
			serial_print_str("TX3.... ");
			data[0] = 'T';
			data[1] = 't';
			its2_router_queue_packet(0x0003, ITS_GENERIC_DATA, &data, 2, 1);
			
			break;
		case 3:
			serial_print_str("Setting address to 1\n");
			mrf24j40_set_extended_address(&EA_1);
			its2_router_init(0x0001, 0x6000);
			
			break;
		case 4:
			serial_print_str("Setting address to 2\n");
			mrf24j40_set_extended_address(&EA_2);
			its2_router_init(0x0002, 0x6000);
			break;
		case 8:
			serial_print_str("Setting address to 3\n");
			mrf24j40_set_extended_address(&EA_3);
			its2_router_init(0x0003, 0x6000);
			break;
		case 5:
			serial_print_str("Setting pan id = 5\n");
			mrf24j40_set_pan_id(0x05);
			break;
		case 6:
			if (promiscuous_mode) {
				serial_print_str("Prom OFF\n");
				promiscuous_mode = 0;
				mrf24j40_short_addr_write(RXMCR, 0x00); 
			} else {
				promiscuous_mode = 1;
				serial_print_str("Prom ON\n");
				mrf24j40_short_addr_write(RXMCR, 0x01); // promiscuous mode, no coord
			}
			break;
		case 10:
				serial_print_str("debug mode\n");
				debug_module = 1;
				break;
		case 11:
				its_print_devices();
				break;
			
		case 7:
			its2_print_queue();
			/*mrf24j40_long_addr_write(0x80, 0xff);
			uns8 val = mrf24j40_long_addr_read(0x80);
			serial_print_str("Read from 0x080 = ");
			serial_print_int_hex(val);
			serial_print_nl(); */
			/*uns8 lowest_channel = mrf24j40_scan_for_lowest_channel_ed();

			serial_print_str("Lowest channel = ");
			serial_print_int(lowest_channel);
			serial_print_nl();	*/	
			break;
	} 	// switch		

	do_task = 0;
}

void print_temp(uns16 temp) {
				
char buffer[8];
uns8 count;
uns8 temp_int;
uns8 temp_dec;
bit negative = 0;

	temp_int = temp >> 8; 

	if (temp.15 == 1) {	// negative
		serial_print_str(" neg ");
		negative = 1;
		temp = temp | 0x0f;	// Set lower 4 bits
		temp = ~temp;	// Take complement
		temp = temp + 1;		// Add one
	}

	temp_int = temp >> 8;
	temp_dec = temp & 0xff;
	temp_dec = temp_dec >> 4;
	temp_dec = convert_to_dec2(temp_dec);
	//serial_print_int(temp_int);
	//serial_print_nl();
	count = 0;
	if (negative) {
		buffer[0] = '-';
		count = 1;
	}
	
	if (temp_int > 9) {
		buffer[count++] = '0' + temp_int / 10;
		buffer[count++] = '0' + temp_int % 10;
	} else {
		buffer[count++] = '0' + temp_int;
	}
	//buffer[count++] = 127; // degree sign
	buffer[count++] = '.';
	if (temp_dec > 9) {
		buffer[count++] = '0' + temp_dec / 10;
		buffer[count++] = '0' + temp_dec % 10;
	} else {
		buffer[count++] = '0';
		buffer[count++] = '0' + temp_dec;
	}


	buffer[count++] = '\0';
	
	serial_print_str("\n<TEMP=");
	serial_print_str(buffer);
	serial_print_str(">");
	serial_print_nl();
	//draw_print_str(0, 6, 16, 0, 3, buffer);
}


void its2_router_receive_callback(uns16 device_id, uns8 *data, uns8 data_length) {
		serial_print_str("Controller receives!");
		if (data_length = 2) {
			uns16 temp = data[1];
			temp = temp << 8;
			temp = temp + data[0];
			print_temp(temp);
		}
}

void main() {
	
	configure_system();
	
	delay_ms(100);
	
	serial_print_str("\n\nITS Mode 1 Controller\n");
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

	its2_router_init(/* device */ 0x1010, /* network id */ 0x5000);
	
	timer_start_0();
	
	while(1) {

		if (serial_rx_avail()) {
			process_rx();
		}

		wpan_process();	
		
		its2_router_process();
		
		handle_tasks();	  

	}	// while (1)

	
}	// main

