/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file usb_cdc_class.h
    \brief  USB Communications Device Class (Serial Port) routines 


*/


#ifndef __usb_cdc_class_h
#define __usb_cdc_class_h

/**
	\brief Sends a single character to the USB serial port
	
	Deliver a single character out the virtual serial port.  This routine will add
	the character to the transmit buffer.  The actual buffer will be physically sent on
	the Start Of Frame (SOF) interrupt (each 1ms) or on the end point interrupt - ie, when the last character or chunk
	of characters was sent.

	\param c The 8 bit byte to be transmitted.
*/
void usb_cdc_putc(uns8 c);

/**
	\brief Retrieve a received character from the USB serial port
	
	Receive a character from the USB serial port.  If a character has
	not been received, this routine will wait indefinately.

	\return Byte from the receive buffer.

*/
uns8 usb_cdc_getc();

/**
	\brief Check to see if a character is available in the receive buffer
	
	If one or more bytes are available in the USB serial port receive buffer, this 
	routine will return true.  If there are no bytes available, it will return false.
	
	\return True if buffer is not empty, False if buffer is empty.
*/
uns8 usb_cdc_rx_avail();

/**
	\brief Check to see if the transmit buffer is empty
	
	Sometimes it is useful to see if the transmit buffer is empty,
	since then you can be sure your data is well on its way.
	In the case of USB, this means that the data has been at least placed
	into the outbound USB buffer; it's not possible to tell until after the fact
	if the data has actually been squirted out the USB port.
	
	\return True if transmit buffer is empty, False if buffer still has data in it.
*/
uns8 usb_cdc_tx_empty();
/**
	\brief Print a string out to the USB virtual serial port
	
	Send a null terminated string out the virtual serial port

	\param str the string to be sent
*/
void usb_cdc_print_str(char *str);

/**
	\brief Print a 16 bit number to the USB virtual serial port
 
	Print a 16 bit unsigned number in decimal to the USB virtual serial port

	\param i the 16 bit number to be printed
*/
void usb_cdc_print_int(uns16 i);

/**
	\brief Transmit any buffered characters over ther USB virtual serial port
 
	Generally only used internally, this routine will attempt to place any
	characters in the transmit queue into the USB buffers. It will fail gracefully
	if the USB buffer is already owned by the SIE.

*/
void usb_cdc_handle_tx();

/**
	\brief Set up data structures ready for USB CDC use
 
	Configures the default DTE rate, stop bits etc.

*/
void usb_cdc_setup();

#endif
